function [output] = vesselmodel(input)
% -------------------------------------------------------------------------
% Vessel model (Manoeuvring model by Andrew Ross [2008])
% -------------------------------------------------------------------------
% > Input vector: [x,u], where x=[eta,nu] and u=[X,Y,K,N]
% > Output vector: [eta_d,nu_d]
% -------------------------------------------------------------------------

% Rename input vector
n   = input(1);
e   = input(2);
phi = 0;
psi = input(3);

u   = input(4);
v   = input(5);
p   = 0;
r   = input(6);

X  = input(7);
Y  = input(8);
K  = 0;
N  = input(9);

eta = [n e psi]';
nu = [u v r]';
tau = [X Y N]';

w = 0; %heave trans vel
q = 0; %pitch ang vel


%% Vessel data
const.rho_water = 1014.0; %water density [kg/m^3]
const.g = 9.81; %gravity constant [m/s^2]
const.deg2rad = pi/180; %degrees to radians
const.rad2deg = 180/pi; %rad to degrees
const.ms2kt = 3600/1852; %m/s to kt
const.kt2ms = 1852/3600; %kt to m/s
const.RPM2rads = 2*pi/60; %RPM to rad/s
const.rads2RPM = 60/(2*pi); %rad/s to RPM
const.HP2W = 745.7; %HP to Watt	

data.Lpp = 4.002; %length between perpendiculars [m]
data.Bwl = 0.538; %maximum beam at the waterline [m]
data.T = 0.172; %vertical distance from the bottom of the hull to the waterline
data.disp = 0.188; %displacement [m^3]
data.m = data.disp*const.rho_water; %mass [kg]
data.Iz =  0.295*data.Lpp; %yaw Inertia
data.Ix = 0.37;%0.37*data.Bwl; %roll Inertia
data.Iy = 0;
data.Ixz = 0;
data.Izx = 0;
data.GMt = 0;
data.rho_g_grad = data.m*const.g;

%% Vessel parameters
vesselparam = [-11.39868418
-228.6143578
-65.2667382
-2.310850598
16.40115857
-324.7102163
-3.54470361
-2.410969557
-100.1205584
-248.4622272
-66.40251546
43.41341206
-167.84629
86.81169441
-143.2693223
193.5840147
-21.38502965
5.495931241
-283.8018589
2346.826376
-2549.299216
-616.9328981
-107.1046813
-993.0229764
-75.0552621
112.5775644
32.13347933
8.858875974
4.6935791
-3.955824974
40.7559673
-608.8345501
69.07718436
110.9176714
-2.784951233
241.3056103
4.986532654
29.90593106
267.5968748
-385.2180129
-51.08433142
-65.43482439
86.32455829
-1081.880365
-109.9285553
-1757.991465
-412.3686809
79.36320704
-207.4414308
-249.0474262];

% Added mass
Xud = vesselparam(1);
Yvd = vesselparam(2);
Ypd = 0;
Yrd = 0;
Kvd = vesselparam(4);
Kpd = 0;
Krd = vesselparam(5);
Nvd = vesselparam(3);
Npd = 0;
Nrd = vesselparam(6);

% Damping forces

% Circulatory Lift and Drag
XuuL = vesselparam(7);
XuuuL = vesselparam(8);
XvvL = vesselparam(9);
XrrL = vesselparam(10);
XrvL = vesselparam(11);
XuvvL = vesselparam(12);
XrvuL = vesselparam(13);
XurrL = vesselparam(14);
XvvppL = 0;
XvrppL = 0;
XrrppL = 0;

YuvL = vesselparam(15);
YurL = vesselparam(16);
YuurL = vesselparam(17);
YuuvL = vesselparam(18);
YvvvL = vesselparam(19);
YrrrL = vesselparam(20);
YrrvL = vesselparam(21);
YvvrL = vesselparam(22);
YuvppL = 0;
YurppL = 0;

KuvL = vesselparam(27);
KurL = vesselparam(28);
KuurL = vesselparam(29);
KuuvL = vesselparam(30);
KvvvL = vesselparam(31);
KrrrL = vesselparam(32);
KrrvL = vesselparam(33);
KvvrL = vesselparam(34);
KuvppL = 0;
KurppL = 0;

NuvL = vesselparam(39);
NurL = vesselparam(40);
NuurL = vesselparam(41);
NuuvL = vesselparam(42);
NvvvL = vesselparam(43);
NrrrL = vesselparam(44);
NrrvL = vesselparam(45);
NvvrL = vesselparam(46);
NuvppL = 0;
NurppL = 0;

% Crossflow Drag
Yvabsv = vesselparam(23);
Yrabsv = vesselparam(26);
Yvabsr = vesselparam(25);
Yrabsr = vesselparam(24);

Kvabsv = vesselparam(35);
Krabsv = vesselparam(38);
Kvabsr = vesselparam(37);
Krabsr = vesselparam(36);

Nvabsv = vesselparam(47);
Nrabsv = vesselparam(50);
Nvabsr = vesselparam(49);
Nrabsr = vesselparam(48);

% Roll Damping
Kp = 0;
Kppp = 0;

%% Low-Frequency Model (5.1)
% -------------------------------------------------------------------------
% > eta_d = J(theta)*nu
% > M*nu_d + C(nu)*nu + D(nu)*nu + g(eta) = tau
% -------------------------------------------------------------------------

Mrb = [data.m 0 0
       0 data.m 0
       0 0 data.Iz];

Ma = -[Xud 0 0
      0 Yvd Yrd
      0 Nvd Nrd];

Ma_ = 1/2*(Ma + Ma');

M = Mrb + Ma_;

Crb = [0 0 data.m*v
       0 0 -data.m*u
       -data.m*v data.m*u  0];

Ca = [0 0 Yvd*v+1/2*(Nvd+Yrd)*r+1/2*(Ypd+Kvd)*p
      0 0 -Xud*u
      -Yvd*v-1/2*(Nvd+Yrd)*r-1/2*(Ypd+Kvd)*p Xud*u 0];
  
Cv = Crb + Ca;

Dv = [-XuuL*u-XuuuL*u^2-XrvuL*r*v -XvvL*v-XrvL*r-XuvvL*u*v-XvvppL*v*phi^2-XvrppL*r*phi^2  -XrrL*r-XurrL*u*r-XrrppL*r*phi^2
     -YuvppL*v*phi^2-YurppL*r*phi^2 -YuvL*u-YuuvL*u^2-YvvvL*v^2-YrrvL*r^2-Yvabsv*abs(v)-Yrabsv*abs(r)  -YurL*u-YuurL*u^2-YrrrL*r^2-YvvrL*v^2-Yvabsr*abs(v)-Yrabsr*abs(r)
     -NuvppL*v*phi^2-NurppL*r*phi^2 -NuvL*u-NuuvL*u^2-NvvvL*v^2-NrrvL*r^2-Nvabsv*abs(v)-Nrabsv*abs(r)  -NurL*u-NuurL*u^2-NrrrL*r^2-NvvrL*v^2-Nvabsr*abs(v)-Nrabsr*abs(r)];

gn = [0
     0
     data.rho_g_grad*data.GMt*sin(phi)
     0];

%% Calculate new states

% Transformation matrix from the {b}-frame to the {n}-frame
cpsi = cos(psi);
spsi = sin(psi);
 
eta_d = [cpsi*u-spsi*v spsi*u+cpsi*v r]';
nu_d = M\(tau - Cv*nu - Dv*nu);

output=[eta_d; nu_d];

